# Copyright (c) 2016 Dave Love, Liverpool University
# MIT licence, per Fedora policy.

Name:          superlu_dist4
Version:       4.3
Release:       2%{?dist}
Summary:       Solution of large, sparse, nonsymmetric systems of linear equations (v4)
License:       BSD
URL:           http://crd-legacy.lbl.gov/~xiaoye/SuperLU/
Source0:       http://crd-legacy.lbl.gov/~xiaoye/SuperLU/superlu_dist_%version.tar.gz
Source1:       superlu_dist-make.inc
# Make the parmetis support work with Scotch
Patch1:        superlu_dist-parmetis.patch
BuildRequires: ptscotch-openmpi-devel scotch-devel openmpi-devel
BuildRequires: ptscotch-openmpi-devel-parmetis gcc-gfortran openblas-devel
# The test program runs if we link with -lmetis but crashes if linked with
# -lscotchmetis.
BuildRequires: metis-devel

%bcond_without mpich
%global mpich mpich
%ifarch %power64
%if 0%{?el6}
%bcond_with mpich
%global mpich %nil
%endif
%endif

%bcond_without openmpi
%ifarch s390 s390x
%bcond_with openmpi
%endif

%if %{with openmpi}
%global openmpi openmpi
%else
%global openmpi %nil
%endif
%if %{with mpich}
%global mpich mpich
%else
%global mpich %nil
%endif

# For library soname.
%global major 0
%global minor 0
%global miner 0
%global sover %major.%minor.%miner


%global desc \
SuperLU is a general purpose library for the direct solution of large,\
sparse, nonsymmetric systems of linear equations.  The library is\
written in C and is callable from either C or Fortran program.  It\
uses MPI, OpenMP and CUDA to support various forms of parallelism.  It\
supports both real and complex datatypes, both single and double\
precision, and 64-bit integer indexing.  The library routines performs\
an LU decomposition with partial pivoting and triangular system solves\
through forward and back substitution.  The LU factorization routines\
can handle non-square matrices but the triangular solves are performed\
only for square matrices.  The matrix columns may be preordered\
(before factorization) either through library or user supplied\
routines.  This preordering for sparsity is completely separate from\
the factorization.  Working precision iterative refinement subroutines\
are provided for improved backward stability.  Routines are also\
provided to equilibrate the system, estimate the condition number,\
calculate the relative backward error, and estimate error bounds for\
the refined solutions.\
\
This version uses MPI and OpenMP.


%description
%desc

%if %{with openmpi}
%package openmpi
Summary:       Solution of large, sparse, nonsymmetric systems of linear equations (v4, openmpi)
Requires:      openmpi%{?_isa}

%description openmpi
%desc

This is the openmpi version.

%package openmpi-devel
Summary: Development files for %name-openmpi
Requires: openmpi-devel%{?_isa}
Requires: %name-openmpi%{?_isa} = %version-%release

%description openmpi-devel
Development files for %name-openmpi
%endif

%package doc
Summary: Documentation for %name
BuildArch: noarch

%description doc
Documentation for %name

%if %{with mpich}
%package mpich
Summary:       Solution of large, sparse, nonsymmetric systems of linear equations (v4, mpich)
BuildRequires: mpich-devel ptscotch-mpich-devel ptscotch-mpich-devel-parmetis
Requires:      mpich%{?_isa}

%description mpich
%desc

This is the mpich version.

%package mpich-devel
Summary: Development files for %name-mpich
Requires: mpich-devel%{?_isa}
Requires: %name-mpich%{?_isa} = %version-%release

%description mpich-devel
Development files for %name-mpich
%endif


%prep
%setup -q -n SuperLU_DIST_%version
%patch1 -p1
cp %SOURCE1 make.inc

%build
# This order to leave openmpi version in place for %%check
for m in %openmpi %mpich; do
case $m in
openmpi) %_openmpi_load ;;
mpich) %_mpich_load ;;
esac
find -name \*.[oa] | xargs rm 2>/dev/null || true # no clean target
pwd
make SuperLUroot=$(pwd)
mkdir -p tmp $m
pushd tmp
ar x ../SRC/libsuperlu_dist.a
mpicc -shared -Wl,-z -Wl,muldefs -Wl,-soname=libsuperlu_dist.so.%major -Wl,--as-needed \
      -o ../$m/libsuperlu_dist.so.%sover *.o -fopenmp -lptscotchparmetis \
      -lptscotch -lptscotcherr -lscotch -lmetis -lopenblas
popd
case $m in
openmpi) %_openmpi_unload
  cp -a EXAMPLE/pddrive pddrive-$m ;;
mpich) %_mpich_unload ;;
esac
done
make -C EXAMPLE clean


%install

for m in %mpich %openmpi; do
case $m in
openmpi) %_openmpi_load ;;
mpich) %_mpich_load ;;
esac
mkdir -p %buildroot$MPI_LIB/%name %buildroot$MPI_INCLUDE/%name
install -m644 SRC/*.h %buildroot$MPI_INCLUDE/%name

install -m 755 $m/libsuperlu_dist.so* %buildroot$MPI_LIB
pushd %buildroot$MPI_LIB
ln -s ../libsuperlu_dist.so.%sover %name/libsuperlu_dist.so
ln -s libsuperlu_dist.so.%sover libsuperlu_dist.so.%major
ln -s libsuperlu_dist.so.%sover libsuperlu_dist.so.%major.%minor
popd
case $m in
openmpi) %_openmpi_unload ;;
mpich) %_mpich_unload ;;
esac
done


%check
# just check that it runs
%_openmpi_load
pushd EXAMPLE
mpirun -n 4 ../pddrive-openmpi -r 2 -c 2 g20.rua


%if %{with openmpi}
%files openmpi
%_libdir/openmpi/lib/*.so.*

%files openmpi-devel
%_libdir/openmpi/lib/%name/*.so
%_includedir/openmpi-%_arch/%name
%endif

%files doc
%doc DOC/ug.pdf EXAMPLE

%if %{with mpich}
%files mpich
%_libdir/mpich/lib/*.so.*

%files mpich-devel
%_libdir/mpich/lib/%name/*.so
%_includedir/mpich-%_arch/%name
%endif


%changelog
* Fri Oct 21 2016 Dave Love <loveshack@fedoraproject.org> - 4.3-2
- Fix soname
- Conditionalize openmpi

* Wed Oct 19 2016 Dave Love <loveshack@fedoraproject.org> - 4.3-1
- Initial version, adapted from the v5 packaging
